@@ -50,19 +50,19 @@
 
 #include "as-utils.h"
 #include "as-utils-private.h"
-#include "as-cache-file.h"
 #include "as-component-private.h"
 #include "as-distro-details.h"
 #include "as-settings-private.h"
 #include "as-distro-extras.h"
 #include "as-stemmer.h"
+#include "as-variant-cache.h"
 
 #include "as-metadata.h"
-#include "as-yamldata.h"
 
 typedef struct
 {
 	GHashTable *cpt_table;
+	GHashTable *known_cids;
 	gchar *screenshot_service_url;
 	gchar *locale;
 	gchar *current_arch;
@@ -75,6 +75,7 @@ typedef struct
 
 	AsPoolFlags flags;
 	AsCacheFlags cache_flags;
+	gboolean prefer_local_metainfo;
 
 	gchar *sys_cache_path;
 	gchar *user_cache_path;
@@ -103,7 +104,10 @@ const gchar *AS_APPSTREAM_METADATA_PATHS[4] = { "/usr/share/app-info",
 #define AS_SEARCH_GREYLIST_STR _("app;application;package;program;programme;suite;tool")
 
 /* where .desktop files are installed to by packages to be registered with the system */
-#define APPLICATIONS_DIR "/usr/share/applications"
+static gchar *APPLICATIONS_DIR = "/usr/share/applications";
+
+/* where metainfo files can be found */
+static gchar *METAINFO_DIR = "/usr/share/metainfo";
 
 static void as_pool_add_metadata_location_internal (AsPool *pool, const gchar *directory, gboolean add_root);
 
@@ -142,10 +146,18 @@ as_pool_init (AsPool *pool)
 	/* set active locale */
 	priv->locale = as_get_current_locale ();
 
+	/* stores known components */
 	priv->cpt_table = g_hash_table_new_full (g_str_hash,
 						g_str_equal,
 						g_free,
 						(GDestroyNotify) g_object_unref);
+
+	/* set which stores whether we have seen a component-ID already */
+	priv->known_cids = g_hash_table_new_full (g_str_hash,
+						  g_str_equal,
+						  g_free,
+						  NULL);
+
 	priv->xml_dirs = g_ptr_array_new_with_free_func (g_free);
 	priv->yaml_dirs = g_ptr_array_new_with_free_func (g_free);
 	priv->icon_dirs = g_ptr_array_new_with_free_func (g_free);
@@ -159,24 +171,35 @@ as_pool_init (AsPool *pool)
 	/* system-wide cache locations */
 	priv->sys_cache_path = g_strdup (AS_APPSTREAM_CACHE_PATH);
 
-	/* users umask shouldn't interfere with us creating new files when we are root */
-	if (as_utils_is_root ())
+	if (as_utils_is_root ()) {
+		/* users umask shouldn't interfere with us creating new files when we are root */
 		as_reset_umask ();
 
+		/* ensure we never start gvfsd as root: https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=852696 */
+		g_setenv ("GIO_USE_VFS", "local", TRUE);
+	}
+
 	/* check the ctime of the cache directory, if it exists at all */
 	as_pool_check_cache_ctime (pool);
 
 	distro = as_distro_details_new ();
 	priv->screenshot_service_url = as_distro_details_get_str (distro, "ScreenshotUrl");
 
+	/* check whether we might want to prefer local metainfo files over remote data */
+	priv->prefer_local_metainfo = as_distro_details_get_bool (distro, "PreferLocalMetainfoData", FALSE);
+
 	/* set watched default directories for AppStream metadata */
 	for (i = 0; AS_APPSTREAM_METADATA_PATHS[i] != NULL; i++)
 		as_pool_add_metadata_location_internal (pool, AS_APPSTREAM_METADATA_PATHS[i], FALSE);
 
 	/* set default pool flags */
 	priv->flags = AS_POOL_FLAG_READ_COLLECTION |
-			AS_POOL_FLAG_READ_METAINFO |
 			AS_POOL_FLAG_READ_DESKTOP_FILES;
+	if (priv->prefer_local_metainfo) {
+		/* FIXME: We don't enable AS_POOL_FLAG_READ_METAINFO by default yet, because this feature is unfinished and needs work,
+		* mainly in the area of merging data together. */
+		priv->flags |= AS_POOL_FLAG_READ_METAINFO;
+	}
 
 	/* set default cache flags */
 	priv->cache_flags = AS_CACHE_FLAG_USE_SYSTEM | AS_CACHE_FLAG_USE_USER;
@@ -193,6 +216,7 @@ as_pool_finalize (GObject *object)
 
 	g_free (priv->screenshot_service_url);
 	g_hash_table_unref (priv->cpt_table);
+	g_hash_table_unref (priv->known_cids);
 
 	g_ptr_array_unref (priv->xml_dirs);
 	g_ptr_array_unref (priv->yaml_dirs);
@@ -219,82 +243,6 @@ as_pool_class_init (AsPoolClass *klass)
 	object_class->finalize = as_pool_finalize;
 }
 
-/**
- * as_merge_components:
- *
- * Merge selected data from two components.
- */
-static void
-as_merge_components (AsComponent *dest_cpt, AsComponent *src_cpt)
-{
-	guint i;
-	AsMergeKind merge_kind;
-
-	merge_kind = as_component_get_merge_kind (src_cpt);
-	g_return_if_fail (merge_kind != AS_MERGE_KIND_NONE);
-
-	/* FIXME: We need to merge more attributes */
-
-	/* merge stuff in append mode */
-	if (merge_kind == AS_MERGE_KIND_APPEND) {
-		GPtrArray *suggestions;
-		GPtrArray *cats;
-
-		/* merge categories */
-		cats = as_component_get_categories (src_cpt);
-		if (cats->len > 0) {
-			g_autoptr(GHashTable) cat_table = NULL;
-			GPtrArray *dest_categories;
-
-			cat_table = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-			for (i = 0; i < cats->len; i++) {
-				const gchar *cat = (const gchar*) g_ptr_array_index (cats, i);
-				g_hash_table_add (cat_table, g_strdup (cat));
-			}
-
-			dest_categories = as_component_get_categories (dest_cpt);
-			if (dest_categories->len > 0) {
-				for (i = 0; i < dest_categories->len; i++) {
-					const gchar *cat = (const gchar*) g_ptr_array_index (dest_categories, i);
-					g_hash_table_add (cat_table, g_strdup (cat));
-				}
-			}
-
-			g_ptr_array_set_size (dest_categories, 0);
-			as_hash_table_string_keys_to_array (cat_table, dest_categories);
-		}
-
-		/* merge suggestions */
-		suggestions = as_component_get_suggested (src_cpt);
-		if (suggestions != NULL) {
-			for (i = 0; i < suggestions->len; i++) {
-				as_component_add_suggested (dest_cpt,
-						    AS_SUGGESTED (g_ptr_array_index (suggestions, i)));
-			}
-		}
-	}
-
-	/* merge stuff in replace mode */
-	if (merge_kind == AS_MERGE_KIND_REPLACE) {
-		gchar **pkgnames;
-
-		/* merge names */
-		if (as_component_get_name (src_cpt) != NULL)
-			as_component_set_name (dest_cpt, as_component_get_name (src_cpt), as_component_get_active_locale (src_cpt));
-
-		/* merge package names */
-		pkgnames = as_component_get_pkgnames (src_cpt);
-		if ((pkgnames != NULL) && (pkgnames[0] != '\0'))
-			as_component_set_pkgnames (dest_cpt, as_component_get_pkgnames (src_cpt));
-
-		/* merge bundles */
-		if (as_component_has_bundle (src_cpt))
-			as_component_set_bundles_array (dest_cpt, as_component_get_bundles (src_cpt));
-	}
-
-	g_debug ("Merged data for '%s'", as_component_get_data_id (dest_cpt));
-}
-
 /**
  * as_pool_add_component_internal:
  * @pool: An instance of #AsPool
@@ -310,6 +258,8 @@ as_pool_add_component_internal (AsPool *pool, AsComponent *cpt, gboolean pedanti
 	const gchar *cdid = NULL;
 	AsComponent *existing_cpt;
 	gint pool_priority;
+	AsOriginKind new_cpt_orig_kind;
+	AsOriginKind existing_cpt_orig_kind;
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
 
 	cdid = as_component_get_data_id (cpt);
@@ -322,6 +272,8 @@ as_pool_add_component_internal (AsPool *pool, AsComponent *cpt, gboolean pedanti
 		return FALSE;
 	}
 
+	new_cpt_orig_kind = as_component_get_origin_kind (cpt);
+
 	existing_cpt = g_hash_table_lookup (priv->cpt_table, cdid);
 	if (as_component_get_origin_kind (cpt) == AS_ORIGIN_KIND_DESKTOP_ENTRY) {
 		g_autofree gchar *tmp_cdid = NULL;
@@ -334,18 +286,79 @@ as_pool_add_component_internal (AsPool *pool, AsComponent *cpt, gboolean pedanti
 			tmp_cdid = g_strdup_printf ("%s.desktop", cdid);
 			existing_cpt = g_hash_table_lookup (priv->cpt_table, tmp_cdid);
 		}
+
+		if (existing_cpt != NULL) {
+			if (as_component_get_origin_kind (cpt) != AS_ORIGIN_KIND_DESKTOP_ENTRY) {
+				/* discard tthis component if we have anything better data already in the pool,
+				 * which is basically anything *but* data from a .desktop file */
+				g_debug ("Ignored .desktop metadata for '%s': Better data exists.", cdid);
+				return FALSE;
+			}
+		}
 	}
 
 	if (existing_cpt == NULL) {
-		/* add additional data to the component, e.g. external screenshots. Also refines
-		* the component's icon paths */
-		as_component_complete (cpt,
-					priv->screenshot_service_url,
-					priv->icon_dirs);
-
 		g_hash_table_insert (priv->cpt_table,
 					g_strdup (cdid),
 					g_object_ref (cpt));
+		g_hash_table_add (priv->known_cids,
+				  g_strdup (as_component_get_id (cpt)));
+		return TRUE;
+	}
+
+	/* safety check so we don't ignore a good component because we added a bad one first */
+	if (!as_component_is_valid (existing_cpt)) {
+		g_debug ("Replacing invalid component '%s' with new one.", cdid);
+		g_hash_table_replace (priv->cpt_table,
+				      g_strdup (cdid),
+				      g_object_ref (cpt));
+		return TRUE;
+	}
+
+	existing_cpt_orig_kind = as_component_get_origin_kind (existing_cpt);
+
+	/* always replace data from .desktop entries */
+	if (existing_cpt_orig_kind == AS_ORIGIN_KIND_DESKTOP_ENTRY) {
+		if (new_cpt_orig_kind == AS_ORIGIN_KIND_METAINFO) {
+			/* do an append-merge to ensure the data from an exusting metainfo file has an icon */
+			as_component_merge_with_mode (cpt,
+							existing_cpt,
+							AS_MERGE_KIND_APPEND);
+
+			g_hash_table_replace (priv->cpt_table,
+				g_strdup (cdid),
+				g_object_ref (cpt));
+			g_debug ("Replaced '%s' with data from metainfo and desktop-entry file.", cdid);
+			return TRUE;
+		} else {
+			as_component_set_priority (existing_cpt, -G_MAXINT);
+		}
+	}
+
+	/* merge desktop-entry data in, if we already have existing data from a metainfo file */
+	if (new_cpt_orig_kind == AS_ORIGIN_KIND_DESKTOP_ENTRY) {
+		if (existing_cpt_orig_kind == AS_ORIGIN_KIND_METAINFO) {
+			/* do an append-merge to ensure the metainfo file has an icon */
+			as_component_merge_with_mode (existing_cpt,
+						      cpt,
+						      AS_MERGE_KIND_APPEND);
+			g_debug ("Merged desktop-entry data into metainfo data for '%s'.", cdid);
+			return TRUE;
+		}
+	}
+
+	/* check whether we should prefer data from metainfo files over preexisting data */
+	if ((priv->prefer_local_metainfo) &&
+	    (new_cpt_orig_kind == AS_ORIGIN_KIND_METAINFO)) {
+		/* update package info, metainfo files do never have this data.
+		 * (we hope that collection data was loaded first here, so the existing_cpt already contains
+		 *  the information we want - if that's not the case, no harm is done here) */
+		as_component_set_pkgnames (cpt, as_component_get_pkgnames (existing_cpt));
+
+		g_hash_table_replace (priv->cpt_table,
+				g_strdup (cdid),
+				g_object_ref (cpt));
+		g_debug ("Replaced '%s' with data from metainfo file.", cdid);
 		return TRUE;
 	}
 
@@ -359,7 +372,7 @@ as_pool_add_component_internal (AsPool *pool, AsComponent *cpt, gboolean pedanti
 							as_component_get_id (cpt));
 		for (i = 0; i < matches->len; i++) {
 			AsComponent *match = AS_COMPONENT (g_ptr_array_index (matches, i));
-			as_merge_components (match, cpt);
+			as_component_merge (match, cpt);
 		}
 
 		return TRUE;
@@ -462,7 +475,7 @@ as_pool_update_addon_info (AsPool *pool, AsComponent *cpt)
 		g_autofree gchar *extended_cdid = NULL;
 		const gchar *extended_cid = (const gchar*) g_ptr_array_index (extends, i);
 
-		extended_cdid = as_utils_build_data_id ("system", "os",
+		extended_cdid = as_utils_build_data_id (AS_COMPONENT_SCOPE_SYSTEM, "os",
 							as_utils_get_component_bundle_kind (cpt),
 							extended_cid);
 
@@ -511,11 +524,24 @@ as_pool_refine_data (AsPool *pool)
 
 		/* validate the component */
 		if (!as_component_is_valid (cpt)) {
-			g_debug ("WARNING: Skipped component '%s': The component is invalid.", as_component_get_id (cpt));
-			ret = FALSE;
+			/* we still succeed if the components originates from a .desktop file -
+			 * we care less about them and they generally have bad quality, so some issues
+			 * pop up on pretty much every system */
+			if (as_component_get_origin_kind (cpt) == AS_ORIGIN_KIND_DESKTOP_ENTRY) {
+				g_debug ("Ignored '%s': The component (from a .desktop file) is invalid.", as_component_get_id (cpt));
+			} else {
+				g_debug ("WARNING: Ignored component '%s': The component is invalid.", as_component_get_id (cpt));
+				ret = FALSE;
+			}
 			continue;
 		}
 
+		/* add additional data to the component, e.g. external screenshots. Also refines
+		* the component's icon paths */
+		as_component_complete (cpt,
+					priv->screenshot_service_url,
+					priv->icon_dirs);
+
 		/* set the "addons" information */
 		as_pool_update_addon_info (pool, cpt);
 
@@ -543,11 +569,19 @@ as_pool_clear (AsPool *pool)
 {
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
 	if (g_hash_table_size (priv->cpt_table) > 0) {
+		/* contents */
 		g_hash_table_unref (priv->cpt_table);
 		priv->cpt_table = g_hash_table_new_full (g_str_hash,
 							 g_str_equal,
 							 g_free,
 							 (GDestroyNotify) g_object_unref);
+
+		/* cid info set */
+		g_hash_table_unref (priv->known_cids);
+		priv->known_cids = g_hash_table_new_full (g_str_hash,
+						  g_str_equal,
+						  g_free,
+						  NULL);
 	}
 }
 
@@ -595,12 +629,12 @@ as_pool_metadata_changed (AsPool *pool)
 }
 
 /**
- * as_pool_load_appstream:
+ * as_pool_load_collection_data:
  *
- * Load fresh metadata from AppStream directories.
+ * Load fresh metadata from AppStream collection data directories.
  */
 static gboolean
-as_pool_load_appstream (AsPool *pool, GError **error)
+as_pool_load_collection_data (AsPool *pool, gboolean refresh, GError **error)
 {
 	GPtrArray *cpts;
 	g_autoptr(GPtrArray) merge_cpts = NULL;
@@ -612,21 +646,23 @@ as_pool_load_appstream (AsPool *pool, GError **error)
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
 
 	/* see if we can use the caches */
-	if (!as_pool_metadata_changed (pool)) {
-		g_autofree gchar *fname = NULL;
-		g_debug ("Caches are up to date.");
+	if (!refresh) {
+		if (!as_pool_metadata_changed (pool)) {
+			g_autofree gchar *fname = NULL;
+			g_debug ("Caches are up to date.");
 
-		if (as_flags_contains (priv->cache_flags, AS_CACHE_FLAG_USE_SYSTEM)) {
-			g_debug ("Using cached data.");
+			if (as_flags_contains (priv->cache_flags, AS_CACHE_FLAG_USE_SYSTEM)) {
+				g_debug ("Using cached data.");
 
-			fname = g_strdup_printf ("%s/%s.gvz", priv->sys_cache_path, priv->locale);
-			if (g_file_test (fname, G_FILE_TEST_EXISTS)) {
-				return as_pool_load_cache_file (pool, fname, error);
+				fname = g_strdup_printf ("%s/%s.gvz", priv->sys_cache_path, priv->locale);
+				if (g_file_test (fname, G_FILE_TEST_EXISTS)) {
+					return as_pool_load_cache_file (pool, fname, error);
+				} else {
+					g_debug ("Missing cache for language '%s', attempting to load fresh data.", priv->locale);
+				}
 			} else {
-				g_debug ("Missing cache for language '%s', attempting to load fresh data.", priv->locale);
+				g_debug ("Not using system cache.");
 			}
-		} else {
-			g_debug ("Not using system cache.");
 		}
 	}
 
@@ -704,9 +740,23 @@ as_pool_load_appstream (AsPool *pool, GError **error)
 			g_error_free (tmp_error);
 			tmp_error = NULL;
 			ret = FALSE;
+
+			if (error != NULL) {
+				if (*error == NULL)
+					g_set_error_literal (error,
+							     AS_POOL_ERROR,
+							     AS_POOL_ERROR_FAILED,
+							     fname);
+				else
+					g_prefix_error (error, "%s, ", fname);
+			}
 		}
 	}
 
+	/* finalize error message, if we had errors */
+	if ((error != NULL) && (*error != NULL))
+		g_prefix_error (error, "%s ", _("Metadata files have errors:"));
+
 	/* add found components to the metadata pool */
 	cpts = as_metadata_get_components (metad);
 	merge_cpts = g_ptr_array_new ();
@@ -746,6 +796,98 @@ as_pool_load_appstream (AsPool *pool, GError **error)
 	return ret;
 }
 
+/**
+ * as_pool_load_metainfo_data:
+ *
+ * Load fresh metadata from metainfo files.
+ */
+static void
+as_pool_load_metainfo_data (AsPool *pool)
+{
+	guint i;
+	g_autoptr(AsMetadata) metad = NULL;
+	g_autoptr(GPtrArray) mi_files = NULL;
+	GPtrArray *cpts;
+	GError *error = NULL;
+	AsPoolPrivate *priv = GET_PRIVATE (pool);
+
+	/* prepare metadata parser */
+	metad = as_metadata_new ();
+	as_metadata_set_locale (metad, priv->locale);
+
+	/* find metainfo files */
+	g_debug ("Searching for data in: %s", METAINFO_DIR);
+	mi_files = as_utils_find_files_matching (METAINFO_DIR, "*.xml", FALSE, NULL);
+	if (mi_files == NULL) {
+		g_debug ("Unable find metainfo files.");
+		return;
+	}
+
+	/* parse the found data */
+	for (i = 0; i < mi_files->len; i++) {
+		g_autoptr(GFile) infile = NULL;
+		const gchar *fname = (const gchar*) g_ptr_array_index (mi_files, i);
+
+		if (!priv->prefer_local_metainfo) {
+			g_autofree gchar *mi_cid = NULL;
+
+			mi_cid = g_path_get_basename (fname);
+			if (g_str_has_suffix (mi_cid, ".metainfo.xml"))
+				mi_cid[strlen (mi_cid) - 13] = '\0';
+			if (g_str_has_suffix (mi_cid, ".appdata.xml")) {
+				g_autofree gchar *mi_cid_desktop = NULL;
+				mi_cid[strlen (mi_cid) - 12] = '\0';
+
+				mi_cid_desktop = g_strdup_printf ("%s.desktop", mi_cid);
+				/* check with .desktop suffix too */
+				if (!g_hash_table_contains (priv->known_cids, mi_cid_desktop)) {
+					g_debug ("Skipped: %s (already known)", fname);
+					continue;
+				}
+			}
+
+			/* quickly check if we know the component already */
+			if (!g_hash_table_contains (priv->known_cids, mi_cid)) {
+				g_debug ("Skipped: %s (already known)", fname);
+				continue;
+			}
+		}
+
+		g_debug ("Reading: %s", fname);
+		infile = g_file_new_for_path (fname);
+		if (!g_file_query_exists (infile, NULL)) {
+			g_warning ("Metadata file '%s' does not exist.", fname);
+			continue;
+		}
+
+		as_metadata_parse_file (metad,
+					infile,
+					AS_FORMAT_KIND_UNKNOWN,
+					&error);
+		if (error != NULL) {
+			g_debug ("WARNING: %s", error->message);
+			g_error_free (error);
+			error = NULL;
+		}
+	}
+
+	/* add found components to the metadata pool */
+	cpts = as_metadata_get_components (metad);
+	for (i = 0; i < cpts->len; i++) {
+		AsComponent *cpt = AS_COMPONENT (g_ptr_array_index (cpts, i));
+
+		/* We only read metainfo files from system directories */
+		as_component_set_scope (cpt, AS_COMPONENT_SCOPE_SYSTEM);
+
+		as_pool_add_component_internal (pool, cpt, FALSE, &error);
+		if (error != NULL) {
+			g_debug ("Metadata ignored: %s", error->message);
+			g_error_free (error);
+			error = NULL;
+		}
+	}
+}
+
 /**
  * as_pool_load_desktop_entries:
  *
@@ -754,10 +896,10 @@ as_pool_load_appstream (AsPool *pool, GError **error)
 static void
 as_pool_load_desktop_entries (AsPool *pool)
 {
-	GPtrArray *cpts;
 	guint i;
 	g_autoptr(AsMetadata) metad = NULL;
 	g_autoptr(GPtrArray) de_files = NULL;
+	GPtrArray *cpts;
 	GError *error = NULL;
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
 
@@ -776,11 +918,30 @@ as_pool_load_desktop_entries (AsPool *pool)
 	/* parse the found data */
 	for (i = 0; i < de_files->len; i++) {
 		g_autoptr(GFile) infile = NULL;
-		const gchar *fname;
+		const gchar *fname = (const gchar*) g_ptr_array_index (de_files, i);
+
+		/* quickly check if we know the component already
+		 * We do not do this when reading metainfo files, since in that case we might
+		 * need to extend their data with .desktop file data. */
+		if (!as_flags_contains (priv->flags, AS_POOL_FLAG_READ_METAINFO)) {
+			g_autofree gchar *de_cid = g_path_get_basename (fname);
+
+			if (!g_hash_table_contains (priv->known_cids, de_cid)) {
+				g_debug ("Skipped: %s (already known)", fname);
+				continue;
+			}
+
+			/* check without .desktop suffix too */
+			if (g_str_has_suffix (de_cid, ".desktop")) {
+				de_cid[strlen (de_cid) - 8] = '\0';
+				if (!g_hash_table_contains (priv->known_cids, de_cid)) {
+					g_debug ("Skipped: %s (already known)", fname);
+					continue;
+				}
+			}
+		}
 
-		fname = (const gchar*) g_ptr_array_index (de_files, i);
 		g_debug ("Reading: %s", fname);
-
 		infile = g_file_new_for_path (fname);
 		if (!g_file_query_exists (infile, NULL)) {
 			g_warning ("Metadata file '%s' does not exist.", fname);
@@ -840,7 +1001,11 @@ as_pool_load (AsPool *pool, GCancellable *cancellable, GError **error)
 
 	/* read all AppStream metadata that we can find */
 	if (as_flags_contains (priv->flags, AS_POOL_FLAG_READ_COLLECTION))
-		ret = as_pool_load_appstream (pool, error);
+		ret = as_pool_load_collection_data (pool, FALSE, error);
+
+	/* read all metainfo files that we can find */
+	if (as_flags_contains (priv->flags, AS_POOL_FLAG_READ_METAINFO))
+		as_pool_load_metainfo_data (pool);
 
 	/* read all .desktop file data that we can find */
 	if (as_flags_contains (priv->flags, AS_POOL_FLAG_READ_DESKTOP_FILES))
@@ -990,7 +1155,7 @@ as_pool_get_components_by_id (AsPool *pool, const gchar *cid)
  * @kind: An #AsProvidesKind
  * @item: The value of the provided item.
  *
- * Find components in the AppStream data pool whcih provide a certain item.
+ * Find components in the AppStream data pool which provide a certain item.
  *
  * Returns: (transfer container) (element-type AsComponent): an array of #AsComponent objects which have been found.
  */
@@ -1103,6 +1268,62 @@ as_pool_get_components_by_categories (AsPool *pool, gchar **categories)
 	return results;
 }
 
+/**
+ * as_pool_get_components_by_launchable:
+ * @pool: An instance of #AsPool.
+ * @kind: An #AsLaunchableKind
+ * @id: The ID of the launchable.
+ *
+ * Find components in the AppStream data pool which provide a specific launchable.
+ * See #AsLaunchable for details on launchables, or refer to the AppStream specification.
+ *
+ * Returns: (transfer container) (element-type AsComponent): an array of #AsComponent objects which have been found.
+ *
+ * Since: 0.11.4
+ */
+GPtrArray*
+as_pool_get_components_by_launchable (AsPool *pool,
+					      AsLaunchableKind kind,
+					      const gchar *id)
+{
+	AsPoolPrivate *priv = GET_PRIVATE (pool);
+	GHashTableIter iter;
+	gpointer value;
+	GPtrArray *results;
+
+	/* sanity check */
+	g_return_val_if_fail (id != NULL, NULL);
+
+	results = g_ptr_array_new_with_free_func (g_object_unref);
+	g_hash_table_iter_init (&iter, priv->cpt_table);
+	while (g_hash_table_iter_next (&iter, NULL, &value)) {
+		GPtrArray *launchables = NULL;
+		guint i;
+		AsComponent *cpt = AS_COMPONENT (value);
+
+		launchables = as_component_get_launchables (cpt);
+		for (i = 0; i < launchables->len; i++) {
+			guint j;
+			GPtrArray *entries;
+			AsLaunchable *launch = AS_LAUNCHABLE (g_ptr_array_index (launchables, i));
+
+			if (kind != AS_LAUNCHABLE_KIND_UNKNOWN) {
+				/* check if the kind matches. an unknown kind matches all provides types */
+				if (as_launchable_get_kind (launch) != kind)
+					continue;
+			}
+
+			entries = as_launchable_get_entries (launch);
+			for (j = 0; j < entries->len; j++) {
+				if (g_strcmp0 ((const gchar*) g_ptr_array_index (entries, j), id) == 0)
+					g_ptr_array_add (results, g_object_ref (cpt));
+			}
+		}
+	}
+
+	return results;
+}
+
 /**
  * as_pool_build_search_terms:
  *
@@ -1253,6 +1474,7 @@ as_pool_refresh_cache (AsPool *pool, gboolean force, GError **error)
 	gboolean ret = FALSE;
 	gboolean ret_poolupdate;
 	g_autofree gchar *cache_fname = NULL;
+	g_autoptr(GError) data_load_error = NULL;
 	g_autoptr(GError) tmp_error = NULL;
 
 	/* try to create cache directory, in case it doesn't exist */
@@ -1297,15 +1519,11 @@ as_pool_refresh_cache (AsPool *pool, gboolean force, GError **error)
 
 	/* NOTE: we will only cache AppStream metadata, no .desktop file metadata etc. */
 
-	/* find them wherever they are */
-	ret = as_pool_load_appstream (pool, &tmp_error);
+	/* load AppStream collection metadata only and refine it */
+	ret = as_pool_load_collection_data (pool, TRUE, &data_load_error);
 	ret_poolupdate = as_pool_refine_data (pool) && ret;
-	if (tmp_error != NULL) {
-		/* the exact error is not forwarded here, since we might be able to partially update the cache */
-		g_warning ("Error while updating the in-memory data pool: %s", tmp_error->message);
-		g_error_free (tmp_error);
-		tmp_error = NULL;
-	}
+	if (data_load_error != NULL)
+		g_debug ("Error while updating the in-memory data pool: %s", data_load_error->message);
 
 	/* save the cache object */
 	as_pool_save_cache_file (pool, cache_fname, &tmp_error);
@@ -1321,10 +1539,16 @@ as_pool_refresh_cache (AsPool *pool, gboolean force, GError **error)
 
 	if (ret) {
 		if (!ret_poolupdate) {
-			g_set_error (error,
+			g_autofree gchar *error_message = NULL;
+			if (data_load_error == NULL)
+				error_message = g_strdup (_("The AppStream system cache was updated, but some errors were detected, which might lead to missing metadata. Refer to the verbose log for more information."));
+			else
+				error_message = g_strdup_printf (_("AppStream system cache was updated, but problems were found: %s"), data_load_error->message);
+
+			g_set_error_literal (error,
 				AS_POOL_ERROR,
 				AS_POOL_ERROR_INCOMPLETE,
-				_("AppStream data pool was loaded, but some metadata was ignored due to errors."));
+				error_message);
 		}
 		/* update the cache mtime, to not needlessly rebuild it again */
 		as_touch_location (cache_fname);
@@ -1333,12 +1557,218 @@ as_pool_refresh_cache (AsPool *pool, gboolean force, GError **error)
 		g_set_error (error,
 				AS_POOL_ERROR,
 				AS_POOL_ERROR_FAILED,
-				_("AppStream cache update failed."));
+				_("AppStream cache update failed. Turn on verbose mode to get more detailed issue information."));
 	}
 
 	return TRUE;
 }
 
+/**
+ * as_cache_file_save:
+ * @fname: The file to save the data to.
+ * @locale: The locale this cache file is for.
+ * @cpts: (element-type AsComponent): The components to serialize.
+ * @error: A #GError
+ *
+ * Serialize components to a cache file and store it on disk.
+ */
+void
+as_cache_file_save (const gchar *fname, const gchar *locale, GPtrArray *cpts, GError **error)
+{
+	g_autoptr(GVariant) main_gv = NULL;
+	g_autoptr(GVariantBuilder) main_builder = NULL;
+	g_autoptr(GVariantBuilder) builder = NULL;
+
+	g_autoptr(GFile) ofile = NULL;
+	g_autoptr(GFileOutputStream) file_out = NULL;
+	g_autoptr(GOutputStream) zout = NULL;
+	g_autoptr(GZlibCompressor) compressor = NULL;
+	gboolean serializable_components_found = FALSE;
+	GError *tmp_error = NULL;
+	guint cindex;
+
+	if (cpts->len == 0) {
+		g_debug ("Skipped writing cache file: No components to serialize.");
+		return;
+	}
+
+	main_builder = g_variant_builder_new (G_VARIANT_TYPE_VARDICT);
+	builder = g_variant_builder_new (G_VARIANT_TYPE_ARRAY);
+
+	for (cindex = 0; cindex < cpts->len; cindex++) {
+		AsComponent *cpt = AS_COMPONENT (g_ptr_array_index (cpts, cindex));
+
+		/* sanity checks */
+		if (!as_component_is_valid (cpt)) {
+			/* we should *never* get here, all invalid stuff should be filtered out at this point */
+			g_critical ("Skipped component '%s' from inclusion into the cache: The component is invalid.",
+					   as_component_get_id (cpt));
+			continue;
+		}
+
+		if (as_component_get_merge_kind (cpt) != AS_MERGE_KIND_NONE) {
+			g_debug ("Skipping '%s' from cache inclusion, it is a merge component.",
+				 as_component_get_id (cpt));
+			continue;
+		}
+		serializable_components_found = TRUE;
+
+		as_component_to_variant (cpt, builder);
+	}
+
+	/* check if we actually have some valid components serialized to a GVariant */
+	if (!serializable_components_found) {
+		g_debug ("Skipped writing cache file: No valid components found for serialization.");
+		return;
+	}
+
+	/* write basic information and add components */
+	g_variant_builder_add (main_builder, "{sv}",
+				"format_version",
+				g_variant_new_uint32 (CACHE_FORMAT_VERSION));
+	g_variant_builder_add (main_builder, "{sv}",
+				"locale",
+				as_variant_mstring_new (locale));
+
+	g_variant_builder_add (main_builder, "{sv}",
+				"components",
+				g_variant_builder_end (builder));
+	main_gv = g_variant_builder_end (main_builder);
+
+	ofile = g_file_new_for_path (fname);
+	compressor = g_zlib_compressor_new (G_ZLIB_COMPRESSOR_FORMAT_GZIP, -1);
+	file_out = g_file_replace (ofile,
+				   NULL, /* entity-tag */
+				   FALSE, /* make backup */
+				   G_FILE_CREATE_REPLACE_DESTINATION,
+				   NULL, /* cancellable */
+				   error);
+	if ((error != NULL) && (*error != NULL))
+		return;
+
+	zout = g_converter_output_stream_new (G_OUTPUT_STREAM (file_out), G_CONVERTER (compressor));
+	if (!g_output_stream_write_all (zout,
+					g_variant_get_data (main_gv),
+					g_variant_get_size (main_gv),
+					NULL, NULL, &tmp_error)) {
+		g_set_error (error,
+			     AS_POOL_ERROR,
+			     AS_POOL_ERROR_FAILED,
+			     "Failed to write stream: %s",
+			     tmp_error->message);
+		g_error_free (tmp_error);
+		return;
+	}
+	if (!g_output_stream_close (zout, NULL, &tmp_error)) {
+		g_set_error (error,
+			     AS_POOL_ERROR,
+			     AS_POOL_ERROR_FAILED,
+			     "Failed to close stream: %s",
+			     tmp_error->message);
+		g_error_free (tmp_error);
+		return;
+	}
+}
+
+/**
+ * as_cache_read:
+ * @fname: The file to save the data to.
+ * @locale: The locale this cache file is for.
+ * @cpts: (element-type AsComponent): The components to serialize.
+ * @error: A #GError
+ *
+ * Serialize components to a cache file and store it on disk.
+ */
+GPtrArray*
+as_cache_file_read (const gchar *fname, GError **error)
+{
+	GPtrArray *cpts = NULL;
+	g_autoptr(GFile) ifile = NULL;
+	g_autoptr(GInputStream) file_stream = NULL;
+	g_autoptr(GInputStream) stream_data = NULL;
+	g_autoptr(GConverter) conv = NULL;
+
+	GByteArray *byte_array;
+	g_autoptr(GBytes) bytes = NULL;
+	gssize len;
+	const gsize buffer_size = 1024 * 32;
+	g_autofree guint8 *buffer = NULL;
+
+	g_autoptr(GVariant) main_gv = NULL;
+	g_autoptr(GVariant) cptsv_array = NULL;
+	g_autoptr(GVariant) cptv = NULL;
+	g_autoptr(GVariant) gmvar = NULL;
+	const gchar *locale = NULL;
+	GVariantIter main_iter;
+
+	ifile = g_file_new_for_path (fname);
+
+	file_stream = G_INPUT_STREAM (g_file_read (ifile, NULL, error));
+	if (file_stream == NULL)
+		return NULL;
+
+	/* decompress the GZip stream */
+	conv = G_CONVERTER (g_zlib_decompressor_new (G_ZLIB_COMPRESSOR_FORMAT_GZIP));
+	stream_data = g_converter_input_stream_new (file_stream, conv);
+
+	buffer = g_malloc (buffer_size);
+	byte_array = g_byte_array_new ();
+	while ((len = g_input_stream_read (stream_data, buffer, buffer_size, NULL, error)) > 0) {
+		g_byte_array_append (byte_array, buffer, len);
+	}
+	bytes = g_byte_array_free_to_bytes (byte_array);
+
+	/* check if there was an error */
+	if (len < 0)
+		return NULL;
+	if ((error != NULL) && (*error != NULL))
+		return NULL;
+
+	main_gv = g_variant_new_from_bytes (G_VARIANT_TYPE_VARDICT, bytes, TRUE);
+	cpts = g_ptr_array_new_with_free_func (g_object_unref);
+
+	gmvar = g_variant_lookup_value (main_gv,
+					"format_version",
+					G_VARIANT_TYPE_UINT32);
+	if ((gmvar == NULL) || (g_variant_get_uint32 (gmvar) != CACHE_FORMAT_VERSION)) {
+		/* don't try to load incompatible cache versions */
+		if (gmvar == NULL)
+			g_warning ("Skipped loading of broken cache file '%s'.", fname);
+		else
+			g_warning ("Skipped loading of incompatible or broken cache file '%s': Format is %i (expected %i)",
+					fname, g_variant_get_uint32 (gmvar), CACHE_FORMAT_VERSION);
+
+		/* TODO: Maybe emit a proper GError? */
+		return NULL;
+	}
+
+	g_variant_unref (gmvar);
+	gmvar = g_variant_lookup_value (main_gv,
+					"locale",
+					G_VARIANT_TYPE_MAYBE);
+	locale = as_variant_get_mstring (&gmvar);
+
+	cptsv_array = g_variant_lookup_value (main_gv,
+					      "components",
+					      G_VARIANT_TYPE_ARRAY);
+
+	g_variant_iter_init (&main_iter, cptsv_array);
+	while ((cptv = g_variant_iter_next_value (&main_iter))) {
+		g_autoptr(AsComponent) cpt = as_component_new ();
+		if (as_component_set_from_variant (cpt, cptv, locale)) {
+			/* add to result list */
+			if (as_component_is_valid (cpt)) {
+				g_ptr_array_add (cpts, g_object_ref (cpt));
+			} else {
+				g_autofree gchar *str = as_component_to_string (cpt);
+				g_warning ("Ignored serialized component: %s", str);
+			}
+		}
+	}
+
+	return cpts;
+}
+
 /**
  * as_pool_set_locale:
  * @pool: An instance of #AsPool.
