@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2014-2016 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2014-2017 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -42,7 +42,6 @@
 #include "as-utils.h"
 #include "as-utils-private.h"
 #include "as-spdx.h"
-#include "as-xmldata.h"
 #include "as-component.h"
 #include "as-component-private.h"
 
@@ -243,6 +242,44 @@ as_validator_check_content_empty (AsValidator *validator, xmlNode *node, const g
 				tag_path);
 }
 
+/**
+ * as_validate_has_hyperlink:
+ *
+ * Check if @text contains a hyperlink.
+ */
+static gboolean
+as_validate_has_hyperlink (const gchar *text)
+{
+	if (text == NULL)
+		return FALSE;
+	if (g_strstr_len (text, -1, "http://") != NULL)
+		return TRUE;
+	if (g_strstr_len (text, -1, "https://") != NULL)
+		return TRUE;
+	if (g_strstr_len (text, -1, "ftp://") != NULL)
+		return TRUE;
+	return FALSE;
+}
+
+/**
+ * as_validate_is_url:
+ *
+ * Check if @str is an URL.
+ */
+static gboolean
+as_validate_is_url (const gchar *str)
+{
+	if (str == NULL)
+		return FALSE;
+	if (g_str_has_prefix (str, "http://"))
+		return TRUE;
+	if (g_str_has_prefix (str, "https://"))
+		return TRUE;
+	if (g_str_has_prefix (str, "ftp://"))
+		return TRUE;
+	return FALSE;
+}
+
 /**
  * as_validator_check_children_quick:
  **/
@@ -304,8 +341,6 @@ static void
 as_validator_check_description_tag (AsValidator *validator, xmlNode* node, AsComponent *cpt, AsFormatStyle mode)
 {
 	xmlNode *iter;
-	gchar *node_content;
-	gchar *node_name;
 	gboolean first_paragraph = TRUE;
 
 	if (mode == AS_FORMAT_STYLE_METAINFO) {
@@ -317,11 +352,12 @@ as_validator_check_description_tag (AsValidator *validator, xmlNode* node, AsCom
 	}
 
 	for (iter = node->children; iter != NULL; iter = iter->next) {
+		const gchar *node_name = (gchar*) iter->name;
+		g_autofree gchar *node_content = (gchar*) xmlNodeGetContent (iter);
+
 		/* discard spaces */
 		if (iter->type != XML_ELEMENT_NODE)
 			continue;
-		node_name = (gchar*) iter->name;
-		node_content = (gchar*) xmlNodeGetContent (iter);
 
 		if ((g_strcmp0 (node_name, "ul") != 0) && (g_strcmp0 (node_name, "ol") != 0)) {
 			as_validator_check_content_empty (validator,
@@ -339,11 +375,11 @@ as_validator_check_description_tag (AsValidator *validator, xmlNode* node, AsCom
 								cpt,
 								"The '%s' tag should not be localized in collection metadata. Localize the whole 'description' tag instead.");
 			}
-			if ((first_paragraph) && (strlen (node_content) < 100)) {
+			if ((first_paragraph) && (strlen (node_content) < 80)) {
 				as_validator_add_issue (validator, iter,
 							AS_ISSUE_IMPORTANCE_INFO,
 							AS_ISSUE_KIND_VALUE_ISSUE,
-							"First 'description/p' paragraph might be too short.",
+							"First 'description/p' paragraph might be too short (< 80 characters).",
 							node_content);
 			}
 			first_paragraph = FALSE;
@@ -373,7 +409,13 @@ as_validator_check_description_tag (AsValidator *validator, xmlNode* node, AsCom
 						node_name);
 		}
 
-		g_free (node_content);
+		if (as_validate_has_hyperlink (node_content)) {
+			as_validator_add_issue (validator, iter,
+						AS_ISSUE_IMPORTANCE_ERROR,
+						AS_ISSUE_KIND_VALUE_WRONG,
+						"The description contains an URL. This is not allowed, please use the <url/> tag to share links.",
+						node_name);
+		}
 	}
 }
 
@@ -484,11 +526,124 @@ as_validator_validate_component_id (AsValidator *validator, xmlNode *idnode, AsC
 	}
 }
 
+/**
+ * as_validator_validate_project_license:
+ */
+static void
+as_validator_validate_project_license (AsValidator *validator, xmlNode *license_node)
+{
+	guint i;
+	g_auto(GStrv) licenses = NULL;
+	g_autofree gchar *license_id = (gchar*) xmlNodeGetContent (license_node);
+
+	licenses = as_spdx_license_tokenize (license_id);
+	if (licenses == NULL) {
+		as_validator_add_issue (validator, license_node,
+					AS_ISSUE_IMPORTANCE_ERROR,
+					AS_ISSUE_KIND_VALUE_WRONG,
+					"SPDX license expression '%s' could not be parsed.",
+					license_id);
+		return;
+	}
+	for (i = 0; licenses[i] != NULL; i++) {
+		if (g_strcmp0 (licenses[i], "&") == 0 ||
+		    g_strcmp0 (licenses[i], "|") == 0 ||
+		    g_strcmp0 (licenses[i], "+") == 0 ||
+		    g_strcmp0 (licenses[i], "(") == 0 ||
+		    g_strcmp0 (licenses[i], ")") == 0)
+			continue;
+		if (licenses[i][0] != '@' ||
+		    !as_is_spdx_license_id (licenses[i] + 1)) {
+			as_validator_add_issue (validator, license_node,
+					AS_ISSUE_IMPORTANCE_WARNING,
+					AS_ISSUE_KIND_VALUE_WRONG,
+					"SPDX license ID '%s' is unknown.",
+					licenses[i]);
+			return;
+		}
+	}
+}
+
+/**
+ * as_validator_validate_update_contact:
+ */
+static void
+as_validator_validate_update_contact (AsValidator *validator, xmlNode *uc_node)
+{
+	g_autofree gchar *text = (gchar*) xmlNodeGetContent (uc_node);
+
+	if ((g_strstr_len (text, -1, "@") == NULL) &&
+	    (g_strstr_len (text, -1, "_at_") == NULL) &&
+	    (g_strstr_len (text, -1, "_AT_") == NULL)) {
+		if (g_strstr_len (text, -1, ".") == NULL) {
+			as_validator_add_issue (validator, uc_node,
+						AS_ISSUE_IMPORTANCE_ERROR,
+						AS_ISSUE_KIND_VALUE_WRONG,
+						"The update-contact '%s' does not appear to be a valid email address.",
+						text);
+		}
+	}
+}
+
+/**
+ * as_validator_check_screenshots:
+ *
+ * Validate a <screenshots/> tag.
+ **/
+static void
+as_validator_check_screenshots (AsValidator *validator, xmlNode *node, AsComponent *cpt)
+{
+	xmlNode *iter;
+	as_validator_check_children_quick (validator, node, "screenshot", cpt);
+
+	for (iter = node->children; iter != NULL; iter = iter->next) {
+		xmlNode *iter2;
+		gboolean image_found = FALSE;
+		gboolean caption_found = FALSE;
+
+		if (iter->type != XML_ELEMENT_NODE)
+			continue;
+
+		for (iter2 = iter->children; iter2 != NULL; iter2 = iter->next) {
+			const gchar *node_name = (const gchar*) iter2->name;
+
+			if (iter2->type != XML_ELEMENT_NODE)
+				continue;
+
+			if (g_strcmp0 (node_name, "image") == 0)
+				image_found = TRUE;
+			else if (g_strcmp0 (node_name, "caption") == 0)
+				caption_found = TRUE;
+			else {
+				as_validator_add_issue (validator, iter2,
+							AS_ISSUE_IMPORTANCE_WARNING,
+							AS_ISSUE_KIND_TAG_UNKNOWN,
+							"Found tag '%s' in a screenshot. Only <cpation/> and <image/> tags are allowed.",
+							(const gchar*) iter2->name);
+			}
+		}
+
+		if (!image_found) {
+			as_validator_add_issue (validator, iter,
+						AS_ISSUE_IMPORTANCE_ERROR,
+						AS_ISSUE_KIND_TAG_MISSING,
+						"The screenshot does not contain any images.");
+		}
+
+		if (!caption_found) {
+			as_validator_add_issue (validator, iter,
+						AS_ISSUE_IMPORTANCE_PEDANTIC,
+						AS_ISSUE_KIND_TAG_MISSING,
+						"The screenshot does not have a caption text.");
+		}
+	}
+}
+
 /**
  * as_validator_validate_component_node:
  **/
 static AsComponent*
-as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xmlNode *root)
+as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xmlNode *root)
 {
 	xmlNode *iter;
 	AsComponent *cpt;
@@ -499,11 +654,11 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 	gboolean has_metadata_license = FALSE;
 
 	found_tags = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-	mode = as_xmldata_get_format_style (xdt);
+	mode = as_context_get_style (ctx);
 
 	/* validate the resulting AsComponent for sanity */
 	cpt = as_component_new ();
-	as_xmldata_parse_component_node (xdt, root, cpt, NULL);
+	as_component_load_from_xml (cpt, ctx, root, NULL);
 	as_validator_set_current_cpt (validator, cpt);
 
 	/* check if component type is valid */
@@ -532,6 +687,33 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 					"The component has a 'merge' method defined. This is not allowed in metainfo files.");
 	}
 
+	/* the component must have an id */
+	if (as_str_empty (as_component_get_id (cpt))) {
+		/* we don't have an id */
+		as_validator_add_issue (validator, NULL,
+					AS_ISSUE_IMPORTANCE_ERROR,
+					AS_ISSUE_KIND_VALUE_MISSING,
+					"The component is missing an ID (<id/> tag).");
+	}
+
+	/* the component must have a name */
+	if (as_str_empty (as_component_get_name (cpt))) {
+		/* we don't have a name */
+		as_validator_add_issue (validator, NULL,
+					AS_ISSUE_IMPORTANCE_ERROR,
+					AS_ISSUE_KIND_VALUE_MISSING,
+					"The component is missing a name (<name/> tag).");
+	}
+
+	/* the component must have a summary */
+	if (as_str_empty (as_component_get_summary (cpt))) {
+		/* we don't have a summary */
+		as_validator_add_issue (validator, NULL,
+					AS_ISSUE_IMPORTANCE_ERROR,
+					AS_ISSUE_KIND_VALUE_MISSING,
+					"The component is missing a summary (<summary/> tag).");
+	}
+
 	for (iter = root->children; iter != NULL; iter = iter->next) {
 		const gchar *node_name;
 		g_autofree gchar *node_content = NULL;
@@ -553,14 +735,6 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 							node_content);
 			}
 			g_free (prop);
-			if (as_component_get_kind (cpt) == AS_COMPONENT_KIND_DESKTOP_APP) {
-				if (!g_str_has_suffix (node_content, ".desktop"))
-					as_validator_add_issue (validator, iter,
-								AS_ISSUE_IMPORTANCE_WARNING,
-								AS_ISSUE_KIND_VALUE_WRONG,
-								"Component id belongs to a desktop-application, but does not resemble the .desktop file name: \"%s\"",
-								node_content);
-			}
 
 			/* validate the AppStream ID */
 			as_validator_validate_component_id (validator, iter, cpt);
@@ -591,9 +765,9 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
 			if (g_str_has_suffix (node_content, ".")) {
 				as_validator_add_issue (validator, iter,
-							AS_ISSUE_IMPORTANCE_INFO,
+							AS_ISSUE_IMPORTANCE_PEDANTIC,
 							AS_ISSUE_KIND_VALUE_ISSUE,
-							"The component name should not end with a \".\" [%s]",
+							"The component name should (likely) not end with a \".\" [%s]",
 							node_content);
 			}
 
@@ -614,6 +788,15 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 							AS_ISSUE_KIND_VALUE_WRONG,
 							"The summary tag must not contain tabs or linebreaks.");
 			}
+
+			if (as_validate_has_hyperlink (summary)) {
+				as_validator_add_issue (validator, iter,
+							AS_ISSUE_IMPORTANCE_ERROR,
+							AS_ISSUE_KIND_VALUE_WRONG,
+							"The summary must not contain any URL.",
+							node_name);
+			}
+
 		} else if (g_strcmp0 (node_name, "description") == 0) {
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
 			as_validator_check_description_tag (validator, iter, cpt, mode);
@@ -621,11 +804,28 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 			gchar *prop;
 			prop = as_validator_check_type_property (validator, cpt, iter);
 			if ((g_strcmp0 (prop, "cached") == 0) || (g_strcmp0 (prop, "stock") == 0)) {
-				if (g_strrstr (node_content, "/") != NULL)
+				if ((g_strrstr (node_content, "/") != NULL) || (as_validate_is_url (node_content)))
 					as_validator_add_issue (validator, iter,
 								AS_ISSUE_IMPORTANCE_ERROR,
 								AS_ISSUE_KIND_VALUE_WRONG,
-								"Icons of type 'stock' or 'cached' must not contain a full or relative path to the icon.");
+								"Icons of type 'stock' or 'cached' must not contain an URL or a full or relative path to the icon.");
+			}
+
+			if (g_strcmp0 (prop, "remote") == 0) {
+				if (!as_validate_is_url (node_content))
+					as_validator_add_issue (validator, iter,
+								AS_ISSUE_IMPORTANCE_ERROR,
+								AS_ISSUE_KIND_VALUE_WRONG,
+								"Icons of type 'remote' must contain an URL to the referenced icon.");
+			}
+
+			if (mode == AS_FORMAT_STYLE_METAINFO) {
+				if ((prop != NULL) && (g_strcmp0 (prop, "stock") != 0)) {
+					as_validator_add_issue (validator, iter,
+								AS_ISSUE_IMPORTANCE_ERROR,
+								AS_ISSUE_KIND_VALUE_WRONG,
+								"Metainfo files may only contain 'stock' icons, icons of kind '%s' are not allowed.", prop);
+				}
 			}
 			g_free (prop);
 		} else if (g_strcmp0 (node_name, "url") == 0) {
@@ -651,13 +851,21 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 		} else if (g_strcmp0 (node_name, "provides") == 0) {
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
 		} else if (g_strcmp0 (node_name, "screenshots") == 0) {
-			as_validator_check_children_quick (validator, iter, "screenshot", cpt);
+			as_validator_check_screenshots (validator, iter, cpt);
 		} else if (g_strcmp0 (node_name, "project_license") == 0) {
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
+			as_validator_validate_project_license (validator, iter);
 		} else if (g_strcmp0 (node_name, "project_group") == 0) {
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
 		} else if (g_strcmp0 (node_name, "developer_name") == 0) {
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
+
+			if (as_validate_has_hyperlink (node_content)) {
+				as_validator_add_issue (validator, iter,
+							AS_ISSUE_IMPORTANCE_WARNING,
+							AS_ISSUE_KIND_VALUE_ISSUE,
+							"The <developer_name/> can not contain a hyperlink.");
+			}
 		} else if (g_strcmp0 (node_name, "compulsory_for_desktop") == 0) {
 			if (!as_utils_is_desktop_environment (node_content)) {
 				as_validator_add_issue (validator, iter,
@@ -681,6 +889,15 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 							AS_ISSUE_KIND_VALUE_WRONG,
 							"Unknown type '%s' for <translation/> tag.", prop);
 			}
+		} else if (g_strcmp0 (node_name, "launchable") == 0) {
+			g_autofree gchar *prop = NULL;
+			prop = as_validator_check_type_property (validator, cpt, iter);
+			if (as_launchable_kind_from_string (prop) == AS_LAUNCHABLE_KIND_UNKNOWN) {
+				as_validator_add_issue (validator, iter,
+							AS_ISSUE_IMPORTANCE_ERROR,
+							AS_ISSUE_KIND_VALUE_WRONG,
+							"Unknown type '%s' for <launchable/> tag.", prop);
+			}
 		} else if (g_strcmp0 (node_name, "extends") == 0) {
 		} else if (g_strcmp0 (node_name, "bundle") == 0) {
 			g_autofree gchar *prop = NULL;
@@ -699,7 +916,15 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 							"The 'update_contact' tag should not be included in collection AppStream XML.");
 			} else {
 				as_validator_check_appear_once (validator, iter, found_tags, cpt);
+				as_validator_validate_update_contact (validator, iter);
 			}
+		} else if (g_strcmp0 (node_name, "suggests") == 0) {
+			as_validator_check_children_quick (validator, iter, "id", cpt);
+		} else if (g_strcmp0 (node_name, "content_rating") == 0) {
+			as_validator_check_children_quick (validator, iter, "content_attribute", cpt);
+		} else if (g_strcmp0 (node_name, "custom") == 0) {
+			as_validator_check_appear_once (validator, iter, found_tags, cpt);
+			as_validator_check_children_quick (validator, iter, "value", cpt);
 		} else if ((g_strcmp0 (node_name, "metadata") == 0) || (g_strcmp0 (node_name, "kudos") == 0)) {
 			/* these tags are GNOME / Fedora specific extensions and are therefore quite common. They shouldn't make the validation fail,
 			 * especially if we might standardize at leat the <kudos/> tag one day, but we should still complain about those tags to make
@@ -707,7 +932,7 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 			as_validator_add_issue (validator, iter,
 						AS_ISSUE_IMPORTANCE_INFO,
 						AS_ISSUE_KIND_TAG_UNKNOWN,
-						"Found invalid tag: '%s'. This tag is a GNOME extensions to AppStream and is not supported by all implementations.",
+						"Found invalid tag: '%s'. This tag is a GNOME-specific extension to AppStream and is not supported by all implementations.",
 						node_name);
 			tag_valid = FALSE;
 		} else if (!g_str_has_prefix (node_name, "x-")) {
@@ -753,10 +978,15 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 					AS_ISSUE_IMPORTANCE_PEDANTIC,
 					AS_ISSUE_KIND_TAG_MISSING,
 					"It would be useful for add a long description to this font to present it better to users.");
-		} else if (cpt_kind != AS_COMPONENT_KIND_GENERIC) {
+		} else if ((cpt_kind == AS_COMPONENT_KIND_DRIVER) || (cpt_kind == AS_COMPONENT_KIND_FIRMWARE)) {
 			as_validator_add_issue (validator, NULL,
 					AS_ISSUE_IMPORTANCE_INFO,
 					AS_ISSUE_KIND_TAG_MISSING,
+					"It is recommended to add a long description to this component to present it better to users.");
+		} else if (cpt_kind != AS_COMPONENT_KIND_GENERIC) {
+			as_validator_add_issue (validator, NULL,
+					AS_ISSUE_IMPORTANCE_PEDANTIC,
+					AS_ISSUE_KIND_TAG_MISSING,
 					"The component is missing a long description. It is recommended to add one.");
 		}
 	}
@@ -774,7 +1004,7 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 	if (as_component_get_kind (cpt) == AS_COMPONENT_KIND_FONT) {
 		if (as_component_get_provided_for_kind (cpt, AS_PROVIDED_KIND_FONT) == NULL)
 			as_validator_add_issue (validator, NULL,
-					AS_ISSUE_IMPORTANCE_WARNING,
+					AS_ISSUE_IMPORTANCE_ERROR,
 					AS_ISSUE_KIND_TAG_MISSING,
 					"Type 'font' component, but no font information was provided via a provides/font tag.");
 	}
@@ -783,7 +1013,7 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 	if (as_component_get_kind (cpt) == AS_COMPONENT_KIND_DRIVER) {
 		if (as_component_get_provided_for_kind (cpt, AS_PROVIDED_KIND_MODALIAS) == NULL)
 			as_validator_add_issue (validator, NULL,
-					AS_ISSUE_IMPORTANCE_INFO,
+					AS_ISSUE_IMPORTANCE_WARNING,
 					AS_ISSUE_KIND_TAG_MISSING,
 					"Type 'driver' component, but no modalias information was provided via a provides/modalias tag.");
 	}
@@ -820,6 +1050,72 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 		}
 	}
 
+	/* validate service specific stuff */
+	if (as_component_get_kind (cpt) == AS_COMPONENT_KIND_SERVICE) {
+		AsLaunchable *launch = as_component_get_launchable (cpt, AS_LAUNCHABLE_KIND_SERVICE);
+		if (launch == NULL || as_launchable_get_entries (launch)->len == 0)
+			as_validator_add_issue (validator, NULL,
+					AS_ISSUE_IMPORTANCE_ERROR,
+					AS_ISSUE_KIND_TAG_MISSING,
+					"This 'service' component is missing a 'launchable' tag of type 'service'.");
+	}
+
+	/* validate suggestions */
+	if (as_component_get_suggested (cpt)->len > 0) {
+		guint j;
+		GPtrArray *sug_array;
+
+		sug_array = as_component_get_suggested (cpt);
+		for (j = 0; j < sug_array->len; j++) {
+			AsSuggested *prov = AS_SUGGESTED (g_ptr_array_index (sug_array, j));
+			if (mode == AS_FORMAT_STYLE_METAINFO) {
+				if (as_suggested_get_kind (prov) != AS_SUGGESTED_KIND_UPSTREAM)
+					as_validator_add_issue (validator, NULL,
+							AS_ISSUE_IMPORTANCE_ERROR,
+							AS_ISSUE_KIND_VALUE_WRONG,
+							"Suggestions of any type other than 'upstream' are not allowed in metainfo files (type was '%s')", as_suggested_kind_to_string (as_suggested_get_kind (prov)));
+			}
+		}
+	}
+
+	/* validate categories */
+	if (as_component_get_categories (cpt)->len > 0) {
+		guint j;
+		GPtrArray *cat_array;
+
+		cat_array = as_component_get_categories (cpt);
+		for (j = 0; j < cat_array->len; j++) {
+			const gchar *category_name = (const gchar*) g_ptr_array_index (cat_array, j);
+
+			if (!as_utils_is_category_name (category_name)) {
+				as_validator_add_issue (validator, NULL,
+							AS_ISSUE_IMPORTANCE_WARNING,
+							AS_ISSUE_KIND_VALUE_WRONG,
+							"The category '%s' defined is not valid. Refer to the Freedesktop menu specification for a list of valid categories.", category_name);
+			}
+		}
+	}
+
+	/* validate screenshots */
+	if (as_component_get_screenshots (cpt)->len > 0) {
+		guint j;
+		GPtrArray *scr_array;
+
+		scr_array = as_component_get_screenshots (cpt);
+		for (j = 0; j < scr_array->len; j++) {
+			AsScreenshot *scr = AS_SCREENSHOT (g_ptr_array_index (scr_array, j));
+			const gchar *scr_caption = as_screenshot_get_caption (scr);
+
+			if ((scr_caption != NULL) && (strlen (scr_caption) > 80)) {
+				as_validator_add_issue (validator, NULL,
+							AS_ISSUE_IMPORTANCE_PEDANTIC,
+							AS_ISSUE_KIND_VALUE_ISSUE,
+							"The screenshot caption '%s' is too long (should be <= 80 characters)",
+							scr_caption);
+			}
+		}
+	}
+
 	as_validator_clear_current_cpt (validator);
 	return cpt;
 }
@@ -902,12 +1198,12 @@ as_validator_validate_file (AsValidator *validator, GFile *metadata_file)
  * as_validator_open_xml_document:
  */
 static xmlDoc*
-as_validator_open_xml_document (AsValidator *validator, AsXMLData *xdt, const gchar *xmldata)
+as_validator_open_xml_document (AsValidator *validator, const gchar *xmldata)
 {
 	xmlDoc *doc;
 	g_autoptr(GError) error = NULL;
 
-	doc = as_xmldata_parse_document (xdt, xmldata, &error);
+	doc = as_xml_parse_document (xmldata, &error);
 	if (doc == NULL) {
 		if (error != NULL) {
 			as_validator_add_issue (validator, NULL,
@@ -935,41 +1231,36 @@ as_validator_validate_data (AsValidator *validator, const gchar *metadata)
 	gboolean ret;
 	xmlNode* root;
 	xmlDoc *doc;
-	g_autoptr(AsXMLData) xdt = NULL;
+	g_autoptr(AsContext) ctx = NULL;
 	AsComponent *cpt;
 
 	/* load the XML data */
-	xdt = as_xmldata_new ();
-	as_xmldata_initialize (xdt, AS_CURRENT_FORMAT_VERSION,
-			       "C",
-				NULL,
-				NULL,
-				NULL,
-				0);
+	ctx = as_context_new ();
+	as_context_set_locale (ctx, "C");
 
-	doc = as_validator_open_xml_document (validator, xdt, metadata);
+	doc = as_validator_open_xml_document (validator, metadata);
 	if (doc == NULL)
 		return FALSE;
 	root = xmlDocGetRootElement (doc);
 
 	ret = TRUE;
 	if (g_strcmp0 ((gchar*) root->name, "component") == 0) {
-		as_xmldata_set_format_style (xdt, AS_FORMAT_STYLE_METAINFO);
-		cpt = as_validator_validate_component_node (validator, xdt, root);
+		as_context_set_style (ctx, AS_FORMAT_STYLE_METAINFO);
+		cpt = as_validator_validate_component_node (validator, ctx, root);
 		if (cpt != NULL)
 			g_object_unref (cpt);
 	} else if (g_strcmp0 ((gchar*) root->name, "components") == 0) {
 		xmlNode *iter;
 		const gchar *node_name;
 
-		as_xmldata_set_format_style (xdt, AS_FORMAT_STYLE_COLLECTION);
+		as_context_set_style (ctx, AS_FORMAT_STYLE_COLLECTION);
 		for (iter = root->children; iter != NULL; iter = iter->next) {
 			/* discard spaces */
 			if (iter->type != XML_ELEMENT_NODE)
 				continue;
 			node_name = (const gchar*) iter->name;
 			if (g_strcmp0 (node_name, "component") == 0) {
-				cpt = as_validator_validate_component_node (validator, xdt, iter);
+				cpt = as_validator_validate_component_node (validator, ctx, iter);
 				if (cpt != NULL)
 					g_object_unref (cpt);
 			} else {
@@ -1068,75 +1359,58 @@ as_validator_analyze_component_metainfo_relation_cb (const gchar *fname, AsCompo
 
 	/* check if the referenced .desktop file exists */
 	if (as_component_get_kind (cpt) == AS_COMPONENT_KIND_DESKTOP_APP) {
-		if (g_hash_table_contains (data->desktop_fnames, as_component_get_desktop_id (cpt))) {
-			g_autofree gchar *desktop_fname_full = NULL;
-			g_autoptr(GKeyFile) dfile = NULL;
-			GError *tmp_error = NULL;
+		AsLaunchable *de_launchable = as_component_get_launchable (cpt, AS_LAUNCHABLE_KIND_DESKTOP_ID);
+		if ((de_launchable != NULL) && (as_launchable_get_entries (de_launchable)->len > 0)) {
+			const gchar *desktop_id = g_ptr_array_index (as_launchable_get_entries (de_launchable), 0);
 
-			desktop_fname_full = g_build_filename (data->apps_dir, as_component_get_desktop_id (cpt), NULL);
-			dfile = g_key_file_new ();
+			if (g_hash_table_contains (data->desktop_fnames, desktop_id)) {
+				g_autofree gchar *desktop_fname_full = NULL;
+				g_autoptr(GKeyFile) dfile = NULL;
+				GError *tmp_error = NULL;
 
-			g_key_file_load_from_file (dfile, desktop_fname_full, G_KEY_FILE_NONE, &tmp_error);
-			if (tmp_error != NULL) {
-				as_validator_add_issue (data->validator, NULL,
-						AS_ISSUE_IMPORTANCE_WARNING,
-						AS_ISSUE_KIND_READ_ERROR,
-						"Unable to read associated .desktop file: %s", tmp_error->message);
-				g_error_free (tmp_error);
-				tmp_error = NULL;
-			} else {
-				/* we successfully opened the .desktop file, now perform some checks */
+				desktop_fname_full = g_build_filename (data->apps_dir, desktop_id, NULL);
+				dfile = g_key_file_new ();
 
-				/* name */
-				if (as_str_empty (as_component_get_name (cpt)) &&
-				    (!g_key_file_has_key (dfile, G_KEY_FILE_DESKTOP_GROUP,
-								 G_KEY_FILE_DESKTOP_KEY_NAME, NULL))) {
-					/* we don't have a summary, and there is also none in the .desktop file - this is bad. */
+				g_key_file_load_from_file (dfile, desktop_fname_full, G_KEY_FILE_NONE, &tmp_error);
+				if (tmp_error != NULL) {
 					as_validator_add_issue (data->validator, NULL,
-							AS_ISSUE_IMPORTANCE_ERROR,
-							AS_ISSUE_KIND_VALUE_MISSING,
-							"The component is missing a name (none found in its metainfo or .desktop file)");
-				}
+							AS_ISSUE_IMPORTANCE_WARNING,
+							AS_ISSUE_KIND_READ_ERROR,
+							"Unable to read associated .desktop file: %s", tmp_error->message);
+					g_error_free (tmp_error);
+					tmp_error = NULL;
+				} else {
+					/* we successfully opened the .desktop file, now perform some checks */
 
-				/* summary */
-				if (as_str_empty (as_component_get_summary (cpt)) &&
-				    (!g_key_file_has_key (dfile, G_KEY_FILE_DESKTOP_GROUP,
-								 G_KEY_FILE_DESKTOP_KEY_COMMENT, NULL))) {
-					/* we don't have a summary, and there is also none in the .desktop file - this is bad. */
-					as_validator_add_issue (data->validator, NULL,
-							AS_ISSUE_IMPORTANCE_ERROR,
-							AS_ISSUE_KIND_VALUE_MISSING,
-							"The component is missing a summary (none found in its metainfo or .desktop file)");
-				}
+					/* categories */
+					if (g_key_file_has_key (dfile, G_KEY_FILE_DESKTOP_GROUP,
+									G_KEY_FILE_DESKTOP_KEY_CATEGORIES, NULL)) {
+						g_autofree gchar *cats_str = NULL;
+						g_auto(GStrv) cats = NULL;
+						guint i;
 
-				/* categories */
-				if (g_key_file_has_key (dfile, G_KEY_FILE_DESKTOP_GROUP,
-								G_KEY_FILE_DESKTOP_KEY_CATEGORIES, NULL)) {
-					g_autofree gchar *cats_str = NULL;
-					g_auto(GStrv) cats = NULL;
-					guint i;
-
-					cats_str = g_key_file_get_string (dfile, G_KEY_FILE_DESKTOP_GROUP,
-										 G_KEY_FILE_DESKTOP_KEY_CATEGORIES, NULL);
-					cats = g_strsplit (cats_str, ";", -1);
-					for (i = 0; cats[i] != NULL; i++) {
-						if (as_str_empty (cats[i]))
-							continue;
-						if (!as_utils_is_category_name (cats[i])) {
-							as_validator_add_issue (data->validator, NULL,
-										AS_ISSUE_IMPORTANCE_ERROR,
-										AS_ISSUE_KIND_VALUE_WRONG,
-										"The category '%s' defined in the .desktop file does not exist.", cats[i]);
+						cats_str = g_key_file_get_string (dfile, G_KEY_FILE_DESKTOP_GROUP,
+											G_KEY_FILE_DESKTOP_KEY_CATEGORIES, NULL);
+						cats = g_strsplit (cats_str, ";", -1);
+						for (i = 0; cats[i] != NULL; i++) {
+							if (as_str_empty (cats[i]))
+								continue;
+							if (!as_utils_is_category_name (cats[i])) {
+								as_validator_add_issue (data->validator, NULL,
+											AS_ISSUE_IMPORTANCE_WARNING,
+											AS_ISSUE_KIND_VALUE_WRONG,
+											"The category '%s' defined in the .desktop file does not exist.", cats[i]);
+							}
 						}
 					}
-				}
 
+				}
+			} else {
+				as_validator_add_issue (data->validator, NULL,
+						AS_ISSUE_IMPORTANCE_ERROR,
+						AS_ISSUE_KIND_FILE_MISSING,
+						"Component metadata refers to a non-existing .desktop file.");
 			}
-		} else {
-			as_validator_add_issue (data->validator, NULL,
-					AS_ISSUE_IMPORTANCE_ERROR,
-					AS_ISSUE_KIND_FILE_MISSING,
-					"Component metadata refers to a non-existing .desktop file.");
 		}
 	}
 
@@ -1164,7 +1438,7 @@ as_validator_validate_tree (AsValidator *validator, const gchar *root_dir)
 	GHashTable *validated_cpts = NULL;
 	guint i;
 	gboolean ret = TRUE;
-	g_autoptr(AsXMLData) xdt = NULL;
+	g_autoptr(AsContext) ctx = NULL;
 	struct MInfoCheckData ht_helper;
 
 	/* cleanup */
@@ -1199,14 +1473,9 @@ as_validator_validate_tree (AsValidator *validator, const gchar *root_dir)
 						g_object_unref);
 
 	/* set up XML parser */
-	xdt = as_xmldata_new ();
-	as_xmldata_initialize (xdt, AS_CURRENT_FORMAT_VERSION,
-			       "C",
-				NULL,
-				NULL,
-				NULL,
-				0);
-	as_xmldata_set_format_style (xdt, AS_FORMAT_STYLE_METAINFO);
+	ctx = as_context_new ();
+	as_context_set_locale (ctx, "C");
+	as_context_set_style (ctx, AS_FORMAT_STYLE_METAINFO);
 
 	/* validate all metainfo files */
 	mfiles = as_utils_find_files_matching (metainfo_dir, "*.xml", FALSE, NULL);
@@ -1284,7 +1553,7 @@ as_validator_validate_tree (AsValidator *validator, const gchar *root_dir)
 		}
 
 		/* now read the XML */
-		doc = as_validator_open_xml_document (validator, xdt, asdata->str);
+		doc = as_validator_open_xml_document (validator, asdata->str);
 		if (doc == NULL) {
 			as_validator_clear_current_fname (validator);
 			continue;
@@ -1294,7 +1563,7 @@ as_validator_validate_tree (AsValidator *validator, const gchar *root_dir)
 		if (g_strcmp0 ((gchar*) root->name, "component") == 0) {
 			AsComponent *cpt;
 			cpt = as_validator_validate_component_node (validator,
-								    xdt,
+								    ctx,
 								    root);
 			if (cpt != NULL)
 				g_hash_table_insert (validated_cpts,
